#include <DS3231.h>
#include "Arduino.h"
#include <Wire.h>

#define DS3231_I2C_ADDR 0x68

// registers of DS3231 Chip
#define DS3231_SECONDS_REG          0x00
#define DS3231_MINUTES_REG          0x01
#define DS3231_HOURS_REG            0x02
#define DS3231_DAYOFMONTH_REG       0x04
#define DS3231_MONTH_REG            0x05
#define DS3231_YEAR_REG             0x06
#define DS3231_TEMPERATURE_REG      0x11

DS3231::DS3231() {
	// nothing to do for this constructor.
}

/***************************************** 
	Public Functions
 *****************************************/

byte DS3231::GET_SECOND () {
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (DS3231_SECONDS_REG);
  Wire.endTransmission ();

  Wire.requestFrom (DS3231_I2C_ADDR, 1);
  return bcdToDec (Wire.read());
  }  // END DS3231_GET_SECOND

byte DS3231::GET_MINUTE () {
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (DS3231_MINUTES_REG);
  Wire.endTransmission ();

  Wire.requestFrom (DS3231_I2C_ADDR, 1);
  return bcdToDec (Wire.read());
} // END DS3231_GET_MINUTE

byte DS3231::GET_HOUR (bool& h12, bool& PM) {
  byte temp_buffer;
  byte hours1;
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (DS3231_HOURS_REG);
  Wire.endTransmission ();

  Wire.requestFrom (DS3231_I2C_ADDR, 1);
  temp_buffer = Wire.read ();
  h12 = temp_buffer & 0b01000000;
  if (h12) {
    PM = temp_buffer & 0b00100000;
    return bcdToDec (temp_buffer & 0b00011111);
  } else {
    return bcdToDec (temp_buffer & 0b00111111);
  }
}  // END DS3231_GET_HOUR

byte DS3231::GET_WEEKDAY () {
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (0x03));
  Wire.endTransmission ();

  Wire.requestFrom (DS3231_I2C_ADDR, 1);
  return bcdToDec (Wire.read());
}

byte DS3231::GET_DAY () {
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (DS3231_DAYOFMONTH_REG));
  Wire.endTransmission ();

  Wire.requestFrom (DS3231_I2C_ADDR, 1);
  return bcdToDec (Wire.read());
}  // END DS3231_GET_DAYOFMONTH

byte DS3231::GET_MONTH (bool& Century) {
  byte temp_buffer;
  byte hour;
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (DS3231_MONTH_REG));
  Wire.endTransmission ();

  Wire.requestFrom (DS3231_I2C_ADDR, 1);
  temp_buffer = Wire.read ();
  Century = temp_buffer & 0b10000000;
  return (bcdToDec (temp_buffer & 0b01111111)) ;
}  // END DS3231_GET_MONTH

byte DS3231::GET_YEAR () {
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (DS3231_YEAR_REG));
  Wire.endTransmission ();

  Wire.requestFrom (DS3231_I2C_ADDR, 1);
  return bcdToDec (Wire.read());
}  // END DS3231_GET_TEMP

float DS3231::GET_TEMPERATURE () {

byte tMSB;
byte tLSB;
float rtcTempC;

Wire.beginTransmission (DS3231_I2C_ADDR); 
Wire.write (DS3231_TEMPERATURE_REG);   // the register where the temp data are stored
Wire.endTransmission ();
Wire.requestFrom (DS3231_I2C_ADDR, 2); // ask for two bytes
if (Wire.available ()) 
{
     tMSB = Wire.read ();      // 2s complement int portion
     tLSB = Wire.read ();       // fraction portion
     rtcTempC = ((((short) tMSB << 8) | (short) tLSB) >> 6) / 4.0; 
}
return rtcTempC;
}  // END DS3231_GET_TEMPERATURE

void DS3231::SET_SECOND (byte Second) {
  // Sets the seconds 
  // This function also resets the Oscillator Stop Flag, which is set
  // whenever power is interrupted.
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (0x00));
  Wire.write (decToBcd (Second)); 
  Wire.endTransmission ();
  // Clear OSF flag
  byte temp_buffer = readControlByte(1);
  writeControlByte ((temp_buffer & 0b01111111), 1);
}  //END DS3231_Set_SECOND

void DS3231::SET_MINUTE (byte Minute) {
  // Sets the minutes 
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (0x01));
  Wire.write (decToBcd (Minute)); 
  Wire.endTransmission ();
} // END DS3231_SET_MINUTE

void DS3231::SET_HOUR (byte Hour) {
  // Sets the hour, without changing 12/24h mode.
  // The hour must be in 24h format.

  bool h12;

  // Start by figuring out what the 12/24 mode is
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (DS3231_HOURS_REG));
  Wire.endTransmission ();
  Wire.requestFrom (DS3231_I2C_ADDR, 1);
  h12 = (Wire.read () & 0b01000000);
  // if h12 is true, it's 12h mode; false is 24h.

  if (h12) {
    // 12 hour
    if (Hour > 12) {
      Hour = decToBcd (Hour-12) | 0b01100000;
    } else {
      Hour = decToBcd (Hour) & 0b11011111;
    }
  } else {
    // 24 hour
    Hour = decToBcd (Hour) & 0b10111111;
  }

  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (DS3231_HOURS_REG));
  Wire.write (Hour);
  Wire.endTransmission ();
}

void DS3231::SET_WEEKDAY (byte DoW) {
  // Sets the Day of Week
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (0x03));
  Wire.write (decToBcd (DoW));  
  Wire.endTransmission ();
}

void DS3231::SET_DAY (byte Date) {
  // Sets the Date
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (DS3231_DAYOFMONTH_REG));
  Wire.write (decToBcd (Date)); 
  Wire.endTransmission ();
}

void DS3231::SET_MONTH (byte Month) {
  // Sets the month
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (DS3231_MONTH_REG));
  Wire.write (decToBcd (Month));  
  Wire.endTransmission ();
}

void DS3231::SET_YEAR (byte Year) {
  // Sets the year
  Wire.beginTransmission (DS3231_I2C_ADDR);
  Wire.write (uint8_t (DS3231_YEAR_REG));
  Wire.write (decToBcd (Year)); 
  Wire.endTransmission ();
}

/***************************************** 
	Private Functions
 *****************************************/

byte DS3231::bcdToDec (byte val) {
  return ( (val/16*10) + (val%16) );
}  // END bcdToDec

byte DS3231::decToBcd (byte val) {
// Convert normal decimal numbers to binary coded decimal
  return ( (val/10*16) + (val%10) );
}

byte DS3231::readControlByte(bool which) {
  // Read selected control byte
  // first byte (0) is 0x0e, second (1) is 0x0f
  Wire.beginTransmission(DS3231_I2C_ADDR);
  if (which) {
    // second control byte
    Wire.write(uint8_t(0x0f));
  } else {
    // first control byte
    Wire.write(uint8_t(0x0e));
  }
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDR, 1);
  return Wire.read(); 
}

void DS3231::writeControlByte(byte control, bool which) {
  // Write the selected control byte.
  // which=false -> 0x0e, true->0x0f.
  Wire.beginTransmission(DS3231_I2C_ADDR);
  if (which) {
    Wire.write(uint8_t(0x0f));
  } else {
    Wire.write(uint8_t(0x0e));
  }
  Wire.write(control);
        Wire.endTransmission();
}